
// Importovanie všetkých tried z balíčka knižnica.
import knižnica.*;

// Import geometrickej triedy Javy, ktorá slúži
// na uchovanie afinnej trasformácie. Potrebujeme
// ju pri práci s SVG tvarmi.
import java.awt.geom.AffineTransform;

// Začiatok definície triedy odvodenej
// od nadradenej triedy GRobot.
public class Asteroidy extends GRobot
{
	// Inštancia obrázka pozadia (do nej prečítame animovaný obrázok):
	private final Obrázok pozadie;

	// Vytvorenie inštancie rakety:
	private final Raketa raketa = new Raketa();

	// Vytvorenie zoznamu striel:
	private final Zoznam<Strela> strely = new Zoznam<Strela>();

	// Vytvorenie zoznamu asteroidov:
	private final Zoznam<Asteroid> asteroidy = new Zoznam<Asteroid>();

	// Súkromný konštruktor.
	private Asteroidy()
	{
		// Úprava veľkosti plátna:
		// super(500, 500);

		// Úprava farebnosti prostredia:
		Svet.farbaPozadia(čierna);
		Svet.farbaPlochy(antracitová);

		// Dočasné vypnutie automatického prekresľovania:
		Svet.nekresli();

		// Vytvorenie zásoby desiatich striel
		// (viac ich v tejto hre nepovolíme):
		for (int i = 0; i < 10; ++i)
			strely.pridaj(new Strela());

		// Koľko asteroidov chceme aktivovať na začiatku:
		int počet = Svet.upravCeléČíslo(3, "Koľko asteroidov?").intValue();

		// Prvé tri asteroidy budú predvolene mieriť na raketu:
		for (int i = 0; i < počet; ++i)
			asteroidy.pridaj(new Asteroid(30, raketa));

		// Vytvorenie zásoby ďalších tridsiatich asteroidov,
		// pričom ak by bolo náhodou treba, tak priebežne
		// môžeme vytvoriť ďalšie:
		for (int i = 0; i < počet * 10; ++i)
			asteroidy.pridaj(new Asteroid());

		// Aktivácia prvých „n“ asteroidov (tých, ktoré
		// mieria na raketu):
		for (int i = 0; i < počet; ++i)
			asteroidy.daj(i).aktivuj();

		// Presunutie rakety na vrch zásobníka kreslenia:
		raketa.naVrch();

		// Prečítanie, spustenie a použitie animovaného obrázka (pozadia):
		// ### pozadie = Obrázok.čítaj("goaldog.gif");
		pozadie = Obrázok.čítaj("pozadie.gif");
		pozadie.spusti();
		vlastnýTvar(pozadie);

		// Skrytie a aktivovanie hlavného robota:
		// ### skry();
		aktivuj();

		// Opätovné zapnutie automatického prekresľovania:
		Svet.kresli();

		// Pridanie položky ponuky Nová hra so skratkou C+N (kde C je
		// ctrl na Windows a command na macOS):
		Svet.pridajPoložkuPonuky("Nová hra", Kláves.VK_N, Kláves.VK_N);
	}

	// Reset aktuálneho stavu – spustenie novej hry:
	private void nováHra()
	{
		// Reset striel:
		for (Strela strela : strely) strela.deaktivuj();

		// Havarovanie rakety bez zvuku (keby sme chceli byť dôslední,
		// definovali by sme novú metódu, ktorá by zabezpečila resetovanie
		// rakety do stavu novej hry):
		raketa.havaruj();

		// Reset asteroidov:
		for (Asteroid asteroid : asteroidy)
		{
			asteroid.deaktivuj();
			asteroid.upravVeľkosť(30);
			asteroid.náhodnáPolohaNaOkraji();
			asteroid.nasmerujNa(raketa);
		}

		// Vypočítame počet aktívnych asteroidov podľa veľkosti zoznamu:
		int počet = asteroidy.size() / 10;

		// Aktivácia prvých „n“ asteroidov:
		for (int i = 0; i < počet; ++i) asteroidy.daj(i).aktivuj();
	}

	// Táto metóda bola vytvorená podľa z dokumentácie.
	// Prečíta tvary zo zadaného SVG súboru do zadanej
	// inštancie SVG podpory.
	public static void prečítajSVG(String súbor, SVGPodpora svgInštancia)
	{
		// Prečítanie tvarov z SVG súboru (ak súbor
		// nejestvuje, prípadne nastane iné zlyhanie,
		// tak bude vrátený počet prečítaných tvarov −1):
		int prečítané = svgInštancia.čítaj(súbor);

		// Ak sa čítanie podarilo, tak presunieme kresbu
		// do stredu súradnicovej sústavy:
		if (-1 != prečítané)
		{
			// Získame súradnice stredu:
			Bod stred = svgInštancia.stredKresby();

			// Vyrobíme transformáciu posunutia, ktorá
			// však očakáva súradnice v rámci súradnicového
			// priestoru Javy (preto neobrátime y-ovú
			// súradnicu):
			AffineTransform posun = AffineTransform.
				getTranslateInstance(-stred.polohaX(), stred.polohaY());

			// Uložíme počet tvarov kresby do pomocnej
			// premennej:
			int počet = svgInštancia.počet();

			// Pridáme transformáciu ku každému tvaru:
			for (int i = 0; i < počet; ++i)
				svgInštancia.pridajTransformácie(i, posun);
		}
	}

	// Táto metóda je prevzatá z dokumentácie. Je mierne
	// upravená. Kreslí tvar zadanej SVG inštancie
	// prostredníctvom zadaného robota.
	public static void vykresliSVG(SVGPodpora svgInštancia, GRobot robot)
	{
		// Inštancia farby sa použije na overenie toho,
		// či má kreslený tvar definovanú farbu výplne
		// alebo čiary. (Ak má, tak farbu priamo použije
		// na vyplnenie alebo nakreslenie čiary.)
		Farba farba;

		// Počet SVG tvarov uložený do samostatnej
		// premennej by mal mierne zvýšiť efektívnosť
		// vykonávania cyklu:
		int počet = svgInštancia.počet();

		// Cyklus – nakreslenie čiar a/alebo výplní
		// všetkých tvarov.
		for (int i = 0; i < počet; ++i)
		{
			// Kreslenie výplne:
			farba = svgInštancia.farbaVýplne(i);
			if (null != farba)
			{
				robot.farba(farba);
				robot.vyplňTvar(svgInštancia.
					dajVýsledný(i), true);
			}

			// Kreslenie čiary:
			farba = svgInštancia.farbaČiary(i);
			if (null != farba)
			{
				double hrúbkaČiary =
					svgInštancia.hrúbkaČiary(i);
				if (Double.isFinite(hrúbkaČiary))
					robot.hrúbkaČiary(hrúbkaČiary);
				else
					robot.hrúbkaČiary(1);
				robot.farba(farba);
				robot.kresliTvar(svgInštancia.
					dajVýsledný(i), true);
			}
		}
	}


	// Táto metóda overí, či má raketa dištanc a ak nie, tak
	// nájde neaktívnu strelu, presunie ju na pozíciu rakety,
	// „skopíruje“ smer rakety a aktivuje ju. (Skrátene povedané –
	// raketa vystrelí…)
	private void vystreľ()
	{
		// Buď „if (!raketa.máDištanc()),“ alebo:
		if (raketa.nemáDištanc())
		{
			for (Strela strela : strely)
			{
				if (strela.neaktívny())
				{
					Svet.zvuk("shot.wav");
					strela.skočNa(raketa);
					strela.smer(raketa);
					strela.aktivuj();
					break;
				}
			}
		}
	}

	// V tejto metóde overíme, či je zadaný (zasiahnutý) asteroid
	// dostatočne veľký na to, aby ešte mohol byť rozbitý na dva
	// menšie. Ak áno, vykonáme to, ak nie, len ho deaktivujeme.
	private void rozbi(Asteroid asteroid)
	{
		if (asteroid.veľkosť() >= 5)
		{
			asteroid.upravVeľkosť(asteroid.veľkosť() / 2);

			for (Asteroid dvojička : asteroidy)
			{
				if (dvojička.neaktívny())
				{
					asteroid.aktivujDvojičku(dvojička);
					break;
				}
			}
		}
		else asteroid.deaktivuj();
	}

	// V reakcii na voľbu položky „slepo“ zareagujeme spustením novej hry.
	// (Pretože do ponuky sme pridali len jednu položku. Keby ich tam bolo
	// viac, museli by sme rozlišovať, ktorá položka bola zvolená.)
	@Override public void voľbaPoložkyPonuky()
	{
		if (ÁNO == Svet.otázka("Želáte si začať novú hru?")) nováHra();
	}

	// V reakcii na aktivitu hlavného robota budeme overovať
	// všetky kolízie – všetky strely s asteroidmi a všetky
	// asteroidy s raketou.
	@Override public void aktivita()
	{
		for (Strela strela : strely)
		{
			if (strela.aktívny())
			{
				for (Asteroid asteroid : asteroidy)
				{
					if (asteroid.aktívny() &&
						strela.koliduje(asteroid))
					{
						Svet.zvuk("boom.wav");
						strela.deaktivuj();
						rozbi(asteroid);
						break;
					}
				}
			}
		}

		// Buď „if (!raketa.máDištanc()),“ alebo:
		if (raketa.nemáDištanc())
		{
			for (Asteroid asteroid : asteroidy)
			{
				if (asteroid.aktívny() &&
					raketa.koliduje(asteroid))
				{
					Svet.zvuk("explode.wav");
					raketa.havaruj();
					rozbi(asteroid);
					break;
				}
			}
		}
	}

	// V tejto reakcii zareagujeme len na uvoľnenie
	// medzerníka, čo bude znamenať vystrelenie.
	@Override public void uvoľnenieKlávesu()
	{
		if (ÚdajeUdalostí.kláves(Kláves.MEDZERA)) vystreľ();
	}

	// Hlavná metóda – tá je spustená pri štarte
	// aplikácie.
	public static void main(String[] args)
	{
		Svet.použiKonfiguráciu("Asteroidy.cfg");

		// Vytvorenie anonymnej inštancie.
		new Asteroidy();
	}
}
